import Taro from '@tarojs/taro'

export default function IMSDK() {
  const STATE = {
    CONNECTING: 0,
    OPEN: 1,
    CLOSING: 2,
    CLOSED: 3,
  }

  const noop = () => []

  const instanceRef = {
    current: undefined,
    onOpen: noop,
    onMessage: noop,
    onError: noop,
    onClose: noop
  }

  const paramsRef = {
    current: {}
  }

  return {
    STATE,
    getState,
    connect,
    listen,
    send,
  }

  function getState() {
    if (instanceRef.current) {
      return instanceRef.current.readyState
    }

    return STATE.CLOSED
  }

  function listen(name, fns) {
    if (typeof fns === 'function') {
      instanceRef[name] = fns
    }
  }

  function connect(options) {
    Object.assign(paramsRef.current, options || {})
    
    if (instanceRef.current && instanceRef.current.readyState === STATE.OPEN) {
      return Promise.resolve()
    }

    return new Promise((resolve, reject) => {
      Taro.connectSocket({
        ...paramsRef.current,
        fail: (err) => reject(err),
      }).then((task) => {
        console.log('[WS][connect]', task)
        instanceRef.current = task
  
        task.onOpen(function () {
          console.log('[WS][onOpen]', paramsRef.current)
          resolve()
        })

        task.onMessage(function (msg) {
          console.info('[WS][onMessage]', msg)
          instanceRef.onMessage().forEach((f) => f(msg))
        })

        task.onError(function (error) {
          console.error('[WS][onError]', error)
          instanceRef.onError().forEach((f) => f(error))
        })

        task.onClose(function (res) {
          console.warn('[WS][onClose]', res)
          instanceRef.onClose().forEach((f) => f(res))
        })
      })
    })
  }

  function send(data) {
    console.log('[WS][SEND]', data)
    return new Promise((resovle, reject) => {
      if (instanceRef.current) {
        instanceRef.current.send({
          data,
          success: resovle,
          fail: (err) => {
            console.error('[WS][SEND]', err)
            reject(err)
          },
        })
      } else {
        reject(STATE.CLOSED)
      }
    })
  }
}
